import {Component, ViewChild} from '@angular/core'
import {currentPermissionsFieldComponent} from '../../../shared/components/permissions-field/permissions-field.component'
import {UserService} from '../../services/user.service'
import {GroupService} from '../../../groups/services/group.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Employee} from '../../../shared/model/employee'
import {AccountService} from '../../../accounts/services/account.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {map} from 'rxjs/operators'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  @ViewChild('permissionsTemplateRef', {static: true}) permissionsTemplateRef

  employee: Employee | null
  formFields: FormField[] = [{
    name: 'id',
    label: 'Numéro d\'utilisateur',
    icon: 'pound',
    type: 'number',
    readonly: true
  }, {
    name: 'firstName',
    label: 'Prénom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un prénom est requis'}
    ]
  }, {
    name: 'lastName',
    label: 'Nom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un nom est requis'}
    ]
  }, {
    name: 'group',
    label: 'Groupe',
    icon: 'account-multiple',
    type: 'select',
    valueFn: employee => employee.group ? employee.group.id : -1,
    options$: this.groupService.allWithDefault.pipe(map(groups => groups.map(g => {
      return {value: g.id, label: g.name}
    })))
  }, {
    name: 'permissions',
    label: 'Permissions',
    type: 'permissions'
  }]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-employee-id',
    consumer: error => this.urlUtils.navigateTo('/admin/user/list')
  }, {
    filter: error => error.type === 'exists-employee-fullName',
    messageProducer: error => `Un utilisateur nommé '${error.fullName}' existe déjà`
  }]

  constructor(
    private accountService: AccountService,
    private employeeService: UserService,
    private groupService: GroupService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.employeeService,
      id,
      employee => this.employee = employee
    )

    this.formFields[this.formFields.length - 1].template = this.permissionsTemplateRef
  }

  submit(values) {
    const permissionsField = currentPermissionsFieldComponent
    if (permissionsField.valid()) {
      this.subscribe(
        this.employeeService.update(
          parseInt(values.id),
          values.firstName,
          values.lastName,
          values.group,
          permissionsField.allEnabledPermissions
        ),
        () => {
          this.urlUtils.navigateTo('/admin/user/list')
        }
      )
    }
  }

  delete() {
    this.subscribeAndNavigate(
      this.employeeService.delete(this.employee.id),
      '/admin/user/list'
    )
  }
}
